@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# Copyright (C) 2001-2010 Peter Selinger.
+# Copyright (C) 2001-2011 Peter Selinger.
 # This file is part of Potrace. It is free software and it is covered
 # by the GNU General Public License. See the file COPYING for details.
 
@@ -158,14 +158,19 @@ action sizetest "$TMP1" 45300 45400
 
 # test gimppath
 action $POTRACE -b gimp -o "$TMP1" "$DATA"
-action filetest "$TMP1" 'Name: Potrace Imported Path' > /dev/null
-action sizetest "$TMP1" 38800 45900
+action filetest "$TMP1" '^<\?xml version="1.0" standalone="no"\?>$' > /dev/null
+action sizetest "$TMP1" 11000 12500
 
 # test xfig
 action $POTRACE -b xfig -o "$TMP1" "$DATA"
 action filetest "$TMP1" '#FIG 3.2' > /dev/null
 action sizetest "$TMP1" 20000 23150
 
+# test dxf
+action $POTRACE -b dxf -o "$TMP1" "$DATA"
+action filetest "$TMP1" '^ *999$' > /dev/null
+action sizetest "$TMP1" 120000 160000
+
 action rm -f "$TMP1"
 
 echo "$NAME: test succeeded" >& 2
