@@ -1,8 +1,7 @@
-/* Copyright (C) 2001-2010 Peter Selinger.
+/* Copyright (C) 2001-2011 Peter Selinger.
    This file is part of Potrace. It is free software and it is covered
    by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: backend_eps.c 227 2010-12-16 05:47:19Z selinger $ */
 
 /* The Postscript backend of Potrace. This can produce "ps" or "eps"
    output, and different kinds of graphical debugging
@@ -680,16 +679,19 @@ static int eps_render(potrace_path_t *plist) {
 static int eps_init(imginfo_t *imginfo) {
   double origx = imginfo->trans.orig[0] + imginfo->lmar;
   double origy = imginfo->trans.orig[1] + imginfo->bmar;
-  double scalex = imginfo->width / imginfo->pixwidth / info.unit;
-  double scaley = imginfo->height / imginfo->pixheight / info.unit;
+  double scalex = imginfo->trans.scalex / info.unit;
+  double scaley = imginfo->trans.scaley / info.unit;
   char *c0, *c1;
 
   shipcom("%%!PS-Adobe-3.0 EPSF-3.0\n");
-  shipcom("%%%%Creator: "POTRACE" "VERSION", written by Peter Selinger 2001-2010\n");
+  shipcom("%%%%Creator: "POTRACE" "VERSION", written by Peter Selinger 2001-2011\n");
   shipcom("%%%%LanguageLevel: %d\n", info.pslevel);
-  shipcom("%%%%BoundingBox: 0 0 %d %d\n", 
-	  (int)ceil(imginfo->trans.bb[0]+imginfo->lmar+imginfo->rmar),
-	  (int)ceil(imginfo->trans.bb[1]+imginfo->tmar+imginfo->bmar));
+  shipcom("%%%%BoundingBox: 0 0 %.0f %.0f\n", 
+	  ceil(imginfo->trans.bb[0]+imginfo->lmar+imginfo->rmar),
+	  ceil(imginfo->trans.bb[1]+imginfo->tmar+imginfo->bmar));
+  shipcom("%%%%HiResBoundingBox: 0 0 %f %f\n", 
+	  imginfo->trans.bb[0]+imginfo->lmar+imginfo->rmar,
+	  imginfo->trans.bb[1]+imginfo->tmar+imginfo->bmar);
   shipcom("%%%%Pages: 1\n");
   shipcom("%%%%EndComments\n");
   
@@ -711,7 +713,7 @@ static int eps_init(imginfo_t *imginfo) {
     ship(debugmacros, info.unit);
   }
   if (origx != 0 || origy != 0) {
-    ship("%.0f %.0f translate\n", origx, origy);
+    ship("%f %f translate\n", origx, origy);
   }
   if (info.angle != 0) {
     ship("%.2f rotate\n", info.angle);
@@ -757,7 +759,7 @@ int init_ps(FILE *fout) {
   eps_callbacks(fout);
 
   shipcom("%%!PS-Adobe-3.0\n");
-  shipcom("%%%%Creator: "POTRACE" "VERSION", written by Peter Selinger 2001-2010\n");
+  shipcom("%%%%Creator: "POTRACE" "VERSION", written by Peter Selinger 2001-2011\n");
   shipcom("%%%%LanguageLevel: %d\n", info.pslevel);
   shipcom("%%%%BoundingBox: 0 0 %d %d\n", info.paperwidth, info.paperheight);
   shipcom("%%%%Pages: (atend)\n");
@@ -800,8 +802,8 @@ int term_ps(FILE *fout) {
 static void eps_pageinit_ps(imginfo_t *imginfo) {
   double origx = imginfo->trans.orig[0] + imginfo->lmar;
   double origy = imginfo->trans.orig[1] + imginfo->bmar;
-  double scalex = imginfo->width / imginfo->pixwidth / info.unit;
-  double scaley = imginfo->height / imginfo->pixheight / info.unit;
+  double scalex = imginfo->trans.scalex / info.unit;
+  double scaley = imginfo->trans.scaley / info.unit;
 
   eps_pagenumber++;
   eps_color = -1;
@@ -810,7 +812,7 @@ static void eps_pageinit_ps(imginfo_t *imginfo) {
   shipcom("%%%%Page: %d %d\n", eps_pagenumber, eps_pagenumber);
   ship("save\n");
   if (origx != 0 || origy != 0) {
-    ship("%.0f %.0f translate\n", origx, origy);
+    ship("%f %f translate\n", origx, origy);
   }
   if (info.angle != 0) {
     ship("%.2f rotate\n", info.angle);
