@@ -1,8 +1,7 @@
-/* Copyright (C) 2001-2010 Peter Selinger.
+/* Copyright (C) 2001-2011 Peter Selinger.
    This file is part of Potrace. It is free software and it is covered
    by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: backend_pgm.c 227 2010-12-16 05:47:19Z selinger $ */
 
 /* The PGM backend of Potrace. Here we custom-render a set of Bezier
    curves and output the result as a greymap. This is merely a
@@ -18,6 +17,7 @@
 #include "render.h"
 #include "main.h"
 #include "auxiliary.h"
+#include "trans.h"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -27,21 +27,6 @@
 #define M_PI 3.14159265358979323846
 #endif
 
-/* structure to hold an affine coordinate transformation */
-struct trans_s {
-  double ox, oy;             /* origin */
-  double dxx, dxy, dyx, dyy; /* transformation matrix */
-};
-typedef struct trans_s trans_t;
-
-static inline dpoint_t trans(dpoint_t p, trans_t t) {
-  dpoint_t res;
-
-  res.x = t.ox + p.x * t.dxx + p.y * t.dyx;
-  res.y = t.oy + p.x * t.dxy + p.y * t.dyy;
-  return res;
-}
-
 static void pgm_path(potrace_curve_t *curve, trans_t t, render_t *rm) {
   dpoint_t *c, c1[3];
   int i;
@@ -74,27 +59,22 @@ int page_pgm(FILE *fout, potrace_path_t *plist, imginfo_t *imginfo) {
   potrace_path_t *p;
   greymap_t *gm;
   render_t *rm;
-  int w = (int)ceil(imginfo->trans.bb[0]+imginfo->lmar+imginfo->rmar);
-  int h = (int)ceil(imginfo->trans.bb[1]+imginfo->tmar+imginfo->bmar);
-  double xs, ys;  /* scaling factors */
-  double si, co;
+  int w, h;
   trans_t t;
   int mode;
-  char *comment = "created by "POTRACE" "VERSION", written by Peter Selinger 2001-2010";
+  char *comment = "created by "POTRACE" "VERSION", written by Peter Selinger 2001-2011";
 
-  si = sin(info.angle/180*M_PI);
-  co = cos(info.angle/180*M_PI);
+  t.bb[0] = imginfo->trans.bb[0]+imginfo->lmar+imginfo->rmar;
+  t.bb[1] = imginfo->trans.bb[1]+imginfo->tmar+imginfo->bmar;
+  t.orig[0] = imginfo->trans.orig[0]+imginfo->lmar;
+  t.orig[1] = imginfo->trans.orig[1]+imginfo->bmar;
+  t.x[0] = imginfo->trans.x[0];
+  t.x[1] = imginfo->trans.x[1];
+  t.y[0] = imginfo->trans.y[0];
+  t.y[1] = imginfo->trans.y[1];
 
-  t.ox = imginfo->trans.orig[0]+imginfo->lmar;
-  t.oy = imginfo->trans.orig[1]+imginfo->bmar;
-
-  xs = imginfo->width / imginfo->pixwidth;
-  ys = imginfo->height / imginfo->pixheight;
-
-  t.dxx = co * xs;
-  t.dxy = si * xs;
-  t.dyx = -si * ys;
-  t.dyy = co * ys;
+  w = (int)ceil(t.bb[0]);
+  h = (int)ceil(t.bb[1]);
 
   gm = gm_new(w, h);
   if (!gm) {
@@ -114,7 +94,7 @@ int page_pgm(FILE *fout, potrace_path_t *plist, imginfo_t *imginfo) {
   render_close(rm);
 
   /* if negative orientation, make sure to invert effect of rendering */
-  mode = xs * ys < 0 ? GM_MODE_NEGATIVE : GM_MODE_POSITIVE;
+  mode = imginfo->width * imginfo->height < 0 ? GM_MODE_NEGATIVE : GM_MODE_POSITIVE;
 
   gm_writepgm(fout, rm->gm, comment, 1, mode, info.gamma);
 
