@@ -1,8 +1,7 @@
-/* Copyright (C) 2001-2010 Peter Selinger.
+/* Copyright (C) 2001-2011 Peter Selinger.
    This file is part of Potrace. It is free software and it is covered
    by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: backend_xfig.c 227 2010-12-16 05:47:19Z selinger $ */
 
 /* The xfig backend of Potrace. */
 
@@ -16,6 +15,7 @@
 #include "potracelib.h"
 #include "lists.h"
 #include "auxiliary.h"
+#include "trans.h"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -71,21 +71,6 @@ static pageformat_t pageformat[] = {
 /* ---------------------------------------------------------------------- */
 /* path-drawing auxiliary functions */
 
-/* structure to hold an affine coordinate transformation */
-struct trans_s {
-  double ox, oy;             /* origin */
-  double dxx, dxy, dyx, dyy; /* transformation matrix */
-};
-typedef struct trans_s trans_t;
-
-static inline dpoint_t trans(dpoint_t p, trans_t t) {
-  dpoint_t res;
-
-  res.x = t.ox + p.x * t.dxx + p.y * t.dyx;
-  res.y = t.oy + p.x * t.dxy + p.y * t.dyy;
-  return res;
-}
-
 /* coordinate quantization */
 static inline point_t unit(dpoint_t p) {
   point_t q;
@@ -192,11 +177,8 @@ static int xfig_get_depth(potrace_path_t *plist) {
 /* public interface for XFIG */
 int page_xfig(FILE *fout, potrace_path_t *plist, imginfo_t *imginfo) {
   trans_t t;
-  double si, co;
   double origx = imginfo->trans.orig[0] + imginfo->lmar;
   double origy = - imginfo->trans.orig[1] - imginfo->bmar + info.paperheight;
-  double scalex = imginfo->width / imginfo->pixwidth;
-  double scaley = imginfo->height / imginfo->pixheight;
   char *formatname;
   int best, penalty;
   pageformat_t *f;
@@ -204,15 +186,12 @@ int page_xfig(FILE *fout, potrace_path_t *plist, imginfo_t *imginfo) {
   int x0, y0, x1, y1;  /* in xfig's coordinates */
   int depth;
   
-  si = sin(info.angle/180*M_PI);
-  co = cos(info.angle/180*M_PI);
-  
-  t.ox = 1200/72.0 * origx;
-  t.oy = 1200/72.0 * origy;
-  t.dxx = 1200/72.0 * scalex * co;
-  t.dxy = 1200/72.0 * scalex * -si;
-  t.dyx = 1200/72.0 * scaley * -si;
-  t.dyy = 1200/72.0 * scaley * -co;
+  t.orig[0] = 1200/72.0 * origx;
+  t.orig[1] = 1200/72.0 * origy;
+  t.x[0] = 1200/72.0 * imginfo->trans.x[0];
+  t.x[1] = -1200/72.0 * imginfo->trans.x[1];
+  t.y[0] = 1200/72.0 * imginfo->trans.y[0];
+  t.y[1] = -1200/72.0 * imginfo->trans.y[1];
 
   x0 = (int)(1200/72.0 * (origx - imginfo->trans.orig[0]));
   y0 = (int)(1200/72.0 * (origy + imginfo->trans.orig[1]));
@@ -236,7 +215,7 @@ int page_xfig(FILE *fout, potrace_path_t *plist, imginfo_t *imginfo) {
 
   /* header */
   fprintf(fout, "#FIG 3.2\n");
-  fprintf(fout, "#created by "POTRACE" "VERSION", written by Peter Selinger 2001-2010\n");
+  fprintf(fout, "#created by "POTRACE" "VERSION", written by Peter Selinger 2001-2011\n");
   fprintf(fout, "Portrait\n");
   fprintf(fout, "Center\n");
   fprintf(fout, "Inches\n");
