@@ -1,8 +1,7 @@
-/* Copyright (C) 2001-2010 Peter Selinger.
+/* Copyright (C) 2001-2011 Peter Selinger.
    This file is part of Potrace. It is free software and it is covered
    by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: main.c 247 2010-12-21 14:54:10Z selinger $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -17,14 +16,15 @@
 #include "backend_eps.h"
 #include "backend_pgm.h"
 #include "backend_svg.h"
-#include "backend_gimp.h"
 #include "backend_xfig.h"
+#include "backend_dxf.h"
 #include "potracelib.h"
 #include "bitmap_io.h"
 #include "bitmap.h"
 #include "platform.h"
 #include "auxiliary.h"
 #include "progress_bar.h"
+#include "trans.h"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -35,7 +35,6 @@
 #endif
 
 #define UNDEF ((double)(1e30))   /* a value to represent "undefined" */
-#define INFTY ((double)(1e30))   /* a value to represent +infinity */
 
 struct info_s info;
 
@@ -99,16 +98,17 @@ struct backend_s {
 typedef struct backend_s backend_t;  
 
 static backend_t backend[] = {
-  {"eps",        ".eps",      0, 0, 0, NULL,    page_eps,     NULL,     1},
-  {"postscript", ".ps",       1, 0, 1, init_ps, page_ps,      term_ps,  1},
-  {"ps",         ".ps",       1, 0, 1, init_ps, page_ps,      term_ps,  1},
-  {"pdf",        ".pdf",      0, 0, 1, init_pdf,page_pdf,     term_pdf, 1},
-  {"pdfpage",    ".pdf",      1, 0, 1, init_pdf,page_pdfpage, term_pdf, 1},
-  {"svg",        ".svg",      0, 0, 0, NULL,    page_svg,     NULL,     1},
-  {"pgm",        ".pgm",      0, 1, 1, NULL,    page_pgm,     NULL,     1},
-  {"gimppath",   ".gimppath", 0, 1, 0, NULL,    page_gimp,    NULL,     1},
-  {"xfig",       ".fig",      1, 0, 0, NULL,    page_xfig,    NULL,     0},
-  {NULL, NULL, 0, 0, 0, NULL, NULL, NULL},
+  { "eps",        ".eps", 0, 0, 0,   NULL,     page_eps,     NULL,     1 },
+  { "postscript", ".ps",  1, 0, 1,   init_ps,  page_ps,      term_ps,  1 },
+  { "ps",         ".ps",  1, 0, 1,   init_ps,  page_ps,      term_ps,  1 },
+  { "pdf",        ".pdf", 0, 0, 1,   init_pdf, page_pdf,     term_pdf, 1 },
+  { "pdfpage",    ".pdf", 1, 0, 1,   init_pdf, page_pdfpage, term_pdf, 1 },
+  { "svg",        ".svg", 0, 0, 0,   NULL,     page_svg,     NULL,     1 },
+  { "dxf",        ".dxf", 0, 1, 0,   NULL,     page_dxf,     NULL,     1 },
+  { "pgm",        ".pgm", 0, 1, 1,   NULL,     page_pgm,     NULL,     1 },
+  { "gimppath",   ".svg", 0, 1, 0,   NULL,     page_gimp,    NULL,     1 },
+  { "xfig",       ".fig", 1, 0, 0,   NULL,     page_xfig,    NULL,     0 },
+  { NULL, NULL, 0, 0, 0, NULL, NULL, NULL, 0 },
 };
 
 /* look up a backend by name. If found, return 0 and set *bp. If not
@@ -198,11 +198,16 @@ static void usage(FILE *f) {
   fprintf(f, " -o, --output <filename>    - write all output to this file\n");
   fprintf(f, " --                         - end of options; 0 or more input filenames follow\n");
   fprintf(f, "Backend selection:\n");
-  fprintf(f, " -e, --eps                  - EPS backend (encapsulated postscript) (default)\n");
-  fprintf(f, " -p, --postscript           - Postscript backend\n");
+  fprintf(f, " -b, --backend <name>       - select backend by name\n");
+  fprintf(f, " -e, --eps                  - EPS backend (encapsulated PostScript) (default)\n");
+  fprintf(f, " -p, --postscript           - PostScript backend\n");
   fprintf(f, " -s, --svg                  - SVG backend (scalable vector graphics)\n");
   fprintf(f, " -g, --pgm                  - PGM backend (portable greymap)\n");
-  fprintf(f, " -b, --backend <name>       - select backend by name\n");
+  fprintf(f, " -b pdf                     - PDF backend (portable document format)\n");
+  fprintf(f, " -b pdfpage                 - fixed page-size PDF backend\n");
+  fprintf(f, " -b dxf                     - DXF backend (drawing interchange format)\n");
+  fprintf(f, " -b gimppath                - Gimppath backend (GNU Gimp)\n");
+  fprintf(f, " -b xfig                    - XFig backend\n");
   fprintf(f, "Algorithm options:\n");
   fprintf(f, " -z, --turnpolicy <policy>  - how to resolve ambiguities in path decomposition\n");
   fprintf(f, " -t, --turdsize <n>         - suppress speckles of up to this size (default 2)\n");
@@ -212,10 +217,11 @@ static void usage(FILE *f) {
   fprintf(f, " -u, --unit <n>             - quantize output to 1/unit pixels (default 10)\n");
   fprintf(f, " -d, --debug <n>            - produce debugging output of type n (n=1,2,3)\n");
   fprintf(f, "Scaling and placement options:\n");
+  fprintf(f, " -P, --pagesize <format>    - page size (default is "DEFAULT_PAPERFORMAT")\n");
   fprintf(f, " -W, --width <dim>          - width of output image\n");
   fprintf(f, " -H, --height <dim>         - height of output image\n");
   fprintf(f, " -r, --resolution <n>[x<n>] - resolution (in dpi)\n");
-  fprintf(f, " -x, --scale <n>[x<n>]      - scaling factor (pgm backend)\n");
+  fprintf(f, " -x, --scale <n>[x<n>]      - scaling factor (pixel-based backends)\n");
   fprintf(f, " -S, --stretch <n>          - yresolution/xresolution\n");
   fprintf(f, " -A, --rotate <angle>       - rotate counterclockwise by angle\n");
   fprintf(f, " -M, --margin <dim>         - margin\n");
@@ -223,13 +229,15 @@ static void usage(FILE *f) {
   fprintf(f, " -R, --rightmargin <dim>    - right margin\n");
   fprintf(f, " -T, --topmargin <dim>      - top margin\n");
   fprintf(f, " -B, --bottommargin <dim>   - bottom margin\n");
-  fprintf(f, "Output options, supported by some backends:\n");
+  fprintf(f, " --tight                    - remove whitespace around the input image\n");
+  fprintf(f, "Color options, supported by some backends:\n");
   fprintf(f, " -C, --color #rrggbb        - set line color (default black)\n");
   fprintf(f, " --fillcolor #rrggbb        - set fill color (default transparent)\n");
   fprintf(f, " --opaque                   - make white shapes opaque\n");
+  fprintf(f, "SVG options:\n");
   fprintf(f, " --group                    - group related paths together\n");
+  fprintf(f, " --flat                     - whole image as a single path\n");
   fprintf(f, "Postscript/EPS/PDF options:\n");
-  fprintf(f, " -P, --pagesize <format>    - page size (default is "DEFAULT_PAPERFORMAT")\n");
   fprintf(f, " -c, --cleartext            - do not compress the output\n");
   fprintf(f, " -2, --level2               - use postscript level 2 compression (default)\n");
 #ifdef HAVE_ZLIB
@@ -246,7 +254,7 @@ static void usage(FILE *f) {
   fprintf(f, " --tty <mode>               - progress bar rendering: vt100 or dumb\n");
   fprintf(f, "\n");
   fprintf(f, "Dimensions can have optional units, e.g. 6.5in, 15cm, 100pt.\n");
-  fprintf(f, "Default is "DEFAULT_DIM_NAME" (or pixels for pgm and gimppath backends).\n");
+  fprintf(f, "Default is "DEFAULT_DIM_NAME" (or pixels for pgm, dxf, and gimppath backends).\n");
   fprintf(f, "Possible input file formats are: pnm (pbm, pgm, ppm), bmp.\n");
   j = fprintf(f, "Backends are: ");
   backend_list(f, j, 78);
@@ -359,11 +367,15 @@ static int parse_color(char *s) {
 /* option processing */
 
 /* codes for options that don't have short form */
-#define OPT_GROUP     300
-#define OPT_OPAQUE    301
-#define OPT_FILLCOLOR 302
-#define OPT_PROGRESS  303
-#define OPT_TTY       304
+enum { 
+  OPT_TIGHT = 300,
+  OPT_FILLCOLOR,
+  OPT_OPAQUE,
+  OPT_GROUP,
+  OPT_FLAT,
+  OPT_PROGRESS,
+  OPT_TTY
+};
 
 static struct option longopts[] = {
   {"help",          0, 0, 'h'},
@@ -380,6 +392,7 @@ static struct option longopts[] = {
   {"rightmargin",   1, 0, 'R'},
   {"topmargin",     1, 0, 'T'},
   {"bottommargin",  1, 0, 'B'},
+  {"tight",         0, 0, OPT_TIGHT},
   {"rotate",        1, 0, 'A'},
   {"pagesize",      1, 0, 'P'},
   {"turdsize",      1, 0, 't'},
@@ -406,6 +419,7 @@ static struct option longopts[] = {
   {"invert",        0, 0, 'i'},
   {"opaque",        0, 0, OPT_OPAQUE},
   {"group",         0, 0, OPT_GROUP},
+  {"flat",          0, 0, OPT_FLAT},
   {"progress",      0, 0, OPT_PROGRESS},
   {"tty",           1, 0, OPT_TTY},
 
@@ -438,6 +452,7 @@ static void dopts(int ac, char *av[]) {
   info.angle = 0;
   info.paperwidth = DEFAULT_PAPERWIDTH;
   info.paperheight = DEFAULT_PAPERHEIGHT;
+  info.tight = 0;
   info.unit = 10;
   info.compress = 1;
   info.pslevel = 2;
@@ -453,7 +468,7 @@ static void dopts(int ac, char *av[]) {
   info.blacklevel = 0.5;
   info.invert = 0;
   info.opaque = 0;
-  info.group = 0;
+  info.grouping = 1;
   info.fillcolor = 0xffffff;
   info.progress = 0;
   info.progress_bar = DEFAULT_PROGRESS_BAR;
@@ -467,13 +482,13 @@ static void dopts(int ac, char *av[]) {
       break;
     case 'v':
     case 'V':
-      fprintf(stdout, ""POTRACE" "VERSION". Copyright (C) 2001-2010 Peter Selinger.\n");
+      fprintf(stdout, ""POTRACE" "VERSION". Copyright (C) 2001-2011 Peter Selinger.\n");
       fprintf(stdout, "Library version: %s\n", potrace_version());
       show_defaults(stdout);
       exit(0);
       break;
     case 'l':
-      fprintf(stdout, ""POTRACE" "VERSION". Copyright (C) 2001-2010 Peter Selinger.\n\n");
+      fprintf(stdout, ""POTRACE" "VERSION". Copyright (C) 2001-2011 Peter Selinger.\n\n");
       license(stdout);
       exit(0);
       break;
@@ -493,13 +508,13 @@ static void dopts(int ac, char *av[]) {
       break;
     case 'r':
       parse_dimensions(optarg, &p, &dimx, &dimy);
-      if (*p == 0 && dimx.d == 0 && dimy.d == 0) {
+      if (*p == 0 && dimx.d == 0 && dimy.d == 0 && dimx.x != 0.0 && dimy.x != 0.0) {
 	info.rx = dimx.x;
 	info.ry = dimy.x;
 	break;
       }
       dim = parse_dimension(optarg, &p);
-      if (*p == 0 && dim.d == 0) {
+      if (*p == 0 && dim.d == 0 && dim.x != 0.0) {
 	info.rx = info.ry = dim.x;
 	break;
       }
@@ -560,6 +575,9 @@ static void dopts(int ac, char *av[]) {
 	exit(1);
       }
       break;
+    case OPT_TIGHT:
+      info.tight = 1;
+      break;
     case 'A':
       info.angle = strtod(optarg, &p);
       if (*p) {
@@ -761,7 +779,10 @@ static void dopts(int ac, char *av[]) {
       info.opaque = 1;
       break;
     case OPT_GROUP:
-      info.group = 1;
+      info.grouping = 2;
+      break;
+    case OPT_FLAT:
+      info.grouping = 0;
       break;
     case OPT_PROGRESS:
       info.progress = 1;
@@ -799,36 +820,12 @@ static void dopts(int ac, char *av[]) {
 /* ---------------------------------------------------------------------- */
 /* calculations with bitmap dimensions, positioning etc */
 
-/* consider a rectangle spanned by the vectors (w,0) and (0,h). Rotate
-   it counterclockwise by angle alpha. Then set the rect_t structure
-   to the resulting rectangle, setting its bounding box, origin,
-   x-basis and y-basis. */
-
-static void rotate_dim(double alpha, double w, double h, rect_t *r) { 
-  double s, c, x0, x1, y0, y1;
-
-  s = sin(alpha/180*M_PI);
-  c = cos(alpha/180*M_PI);
-
-  /* apply the transformation matrix to the basis vectors */
-  x0 = c*w;
-  x1 = s*w;
-  y0 = -s*h;
-  y1 = c*h;
-
-  /* determine bounding box and origin relative to bounding box */
-  r->bb[0] = fabs(x0) + fabs(y0);
-  r->bb[1] = fabs(x1) + fabs(y1);
-  r->orig[0] = - min(x0,0) - min(y0,0);
-  r->orig[1] = - min(x1,0) - min(y1,0);
-}
-  
 /* determine the dimensions of the output based on command line and
-   image dimensions */
-static void calc_dimensions(imginfo_t *imginfo) {
+   image dimensions, and optionally, based on the actual image outline. */
+static void calc_dimensions(imginfo_t *imginfo, potrace_path_t *plist) {
   double dim_def;
   double maxwidth, maxheight, sc;
-  rect_t r;
+  int default_scaling = 0;
 
   /* we take care of a special case: if one of the image dimensions is
      0, we change it to 1. Such an image is empty anyway, so there
@@ -860,78 +857,94 @@ static void calc_dimensions(imginfo_t *imginfo) {
   imginfo->tmar = info.tmar_d.x == UNDEF ? UNDEF : double_of_dim(info.tmar_d, dim_def);
   imginfo->bmar = info.bmar_d.x == UNDEF ? UNDEF : double_of_dim(info.bmar_d, dim_def);
 
-  /* determine width and height from desired resolution / scaling
-     factor, if given */
+  /* start with a standard rectangle */
+  trans_from_rect(&imginfo->trans, imginfo->pixwidth, imginfo->pixheight);
+
+  /* if info.tight is set, tighten the bounding box */
+  if (info.tight) {
+    trans_tighten(&imginfo->trans, plist);
+  }
+
+  /* sx/rx is just an alternate way to specify width; sy/ry is just an
+     alternate way to specify height. */
   if (info.backend->pixel) {
     if (imginfo->width == UNDEF && info.sx != UNDEF) {
-      imginfo->width = imginfo->pixwidth * info.sx;
+      imginfo->width = imginfo->trans.bb[0] * info.sx;
     }
     if (imginfo->height == UNDEF && info.sy != UNDEF) {
-      imginfo->height = imginfo->pixheight * info.sy;
+      imginfo->height = imginfo->trans.bb[1] * info.sy;
     }
   } else {
     if (imginfo->width == UNDEF && info.rx != UNDEF) {
-      imginfo->width = imginfo->pixwidth / info.rx * 72;
+      imginfo->width = imginfo->trans.bb[0] / info.rx * 72;
     }
     if (imginfo->height == UNDEF && info.ry != UNDEF) {
-      imginfo->height = imginfo->pixheight / info.ry * 72;
+      imginfo->height = imginfo->trans.bb[1] / info.ry * 72;
     }
   }
 
-  /* if one of width/height is specified, determine the other */
+  /* if one of width/height is specified, use stretch to determine the
+     other */
   if (imginfo->width == UNDEF && imginfo->height != UNDEF) {
-    imginfo->width = imginfo->height / imginfo->pixheight * imginfo->pixwidth / info.stretch;
+    imginfo->width = imginfo->height / imginfo->trans.bb[1] * imginfo->trans.bb[0] / info.stretch;
   } else if (imginfo->width != UNDEF && imginfo->height == UNDEF) {
-    imginfo->height = imginfo->width / imginfo->pixwidth * imginfo->pixheight * info.stretch;
+    imginfo->height = imginfo->width / imginfo->trans.bb[0] * imginfo->trans.bb[1] * info.stretch;
   }
 
-  /* if width and height are still variable, figure them out */
+  /* if width and height are still variable, tenatively use the
+     default scaling factor of 72dpi (for dimension-based backends) or
+     1 (for pixel-based backends). For fixed-size backends, this will
+     be adjusted later to fit the page. */
   if (imginfo->width == UNDEF && imginfo->height == UNDEF) {
-   
-    if (info.backend->fixed) {
+    imginfo->width = imginfo->trans.bb[0];
+    imginfo->height = imginfo->trans.bb[1] * info.stretch;
+    default_scaling = 1;
+  } 
 
-      /* in fixed-size backends, try to squeeze it between margins */
-      maxwidth = UNDEF;
-      maxheight = UNDEF;
-      
-      if (imginfo->lmar != UNDEF && imginfo->rmar != UNDEF) {
-	maxwidth = info.paperwidth - imginfo->lmar - imginfo->rmar;
-      } 
-      if (imginfo->bmar != UNDEF && imginfo->tmar != UNDEF) {
-	maxheight = info.paperheight - imginfo->bmar - imginfo->tmar;
-      }
-      if (maxwidth == UNDEF && maxheight == UNDEF) {
-	maxwidth = max(info.paperwidth - 144, info.paperwidth * 0.75);
-	maxheight = max(info.paperheight - 144, info.paperheight * 0.75);
-      }
-      
-      rotate_dim(info.angle, imginfo->pixwidth, imginfo->pixheight * info.stretch, &r);
-      
-      sc = min(maxwidth == UNDEF ? INFTY : maxwidth / r.bb[0], maxheight == UNDEF ? INFTY : maxheight / r.bb[1]);
-      imginfo->width = imginfo->pixwidth * sc;
-      imginfo->height = imginfo->pixheight * info.stretch * sc;
+  /* apply scaling */
+  trans_scale_to_size(&imginfo->trans, imginfo->width, imginfo->height);
 
-    } else if (info.backend->pixel) {
-
-      /* in pixel-based backends, assume default scaling factor of 1 */
-
-      imginfo->width = imginfo->pixwidth;
-      imginfo->height = imginfo->pixheight * info.stretch;
-    } else {
-
-      /* otherwise, choose a default size based on the default paper format */
+  /* apply rotation, and tighten the bounding box again, if necessary */
+  if (info.angle != 0.0) {
+    trans_rotate(&imginfo->trans, info.angle);
+    if (info.tight) {
+      trans_tighten(&imginfo->trans, plist);
+    }
+  }
 
+  /* for fixed-size backends, if default scaling was in effect,
+     further adjust the scaling to be the "best fit" for the given
+     page size and margins. */
+  if (default_scaling && info.backend->fixed) {
+    
+    /* try to squeeze it between margins */
+    maxwidth = UNDEF;
+    maxheight = UNDEF;
+    
+    if (imginfo->lmar != UNDEF && imginfo->rmar != UNDEF) {
+      maxwidth = info.paperwidth - imginfo->lmar - imginfo->rmar;
+    } 
+    if (imginfo->bmar != UNDEF && imginfo->tmar != UNDEF) {
+      maxheight = info.paperheight - imginfo->bmar - imginfo->tmar;
+    }
+    if (maxwidth == UNDEF && maxheight == UNDEF) {
       maxwidth = max(info.paperwidth - 144, info.paperwidth * 0.75);
       maxheight = max(info.paperheight - 144, info.paperheight * 0.75);
-      
-      sc = min(maxwidth / imginfo->pixwidth, maxheight / imginfo->pixheight / info.stretch);
-      imginfo->width = imginfo->pixwidth * sc;
-      imginfo->height = imginfo->pixheight * info.stretch * sc;
     }
-  }
+    
+    if (maxwidth == UNDEF) {
+      sc = maxheight / imginfo->trans.bb[1];
+    } else if (maxheight == UNDEF) {
+      sc = maxwidth / imginfo->trans.bb[0];
+    } else {
+      sc = min(maxwidth / imginfo->trans.bb[0], maxheight / imginfo->trans.bb[1]);
+    }
 
-  /* calculate coordinate system */
-  rotate_dim(info.angle, imginfo->width, imginfo->height, &imginfo->trans);
+    /* re-scale coordinate system */
+    imginfo->width *= sc;
+    imginfo->height *= sc;
+    trans_rescale(&imginfo->trans, sc);
+  }
 
   /* adjust margins */
   if (info.backend->fixed) {
@@ -1090,10 +1103,10 @@ static void process_file(backend_t *b, const char *infile, const char *outfile,
     /* calculate image dimensions */
     imginfo.pixwidth = bm->w;
     imginfo.pixheight = bm->h;
-    calc_dimensions(&imginfo);
-
     bm_free(bm);
 
+    calc_dimensions(&imginfo, st->plist);
+
     r = b->page_f(fout, st->plist, &imginfo);
     if (r) {
       fprintf(stderr, ""POTRACE": %s: %s\n", outfile, strerror(errno));
