@@ -1,8 +1,7 @@
-/* Copyright (C) 2001-2010 Peter Selinger.
+/* Copyright (C) 2001-2011 Peter Selinger.
    This file is part of Potrace. It is free software and it is covered
    by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: main.h 237 2010-12-20 05:16:20Z selinger $ */
 
 #ifndef MAIN_H
 #define MAIN_H
@@ -13,13 +12,8 @@
 
 #include "potracelib.h"
 #include "progress_bar.h"
-
-/* structure to hold a tilted rectangle */
-struct rect_s {
-  double bb[2];    /* dimensions of bounding box */
-  double orig[2];  /* origin relative to bounding box */
-};
-typedef struct rect_s rect_t;
+#include "auxiliary.h"
+#include "trans.h"
 
 /* structure to hold a dimensioned value */
 struct dim_s {
@@ -78,6 +72,7 @@ struct info_s {
   dim_t lmar_d, rmar_d, tmar_d, bmar_d;   /* margins */
   double angle;      /* rotate by this many degrees */
   int paperwidth, paperheight;  /* paper size for ps backend (in pt) */
+  int tight;         /* should bounding box follow actual vector outline? */
   double unit;       /* granularity of output grid */
   int compress;      /* apply compression? */
   int pslevel;       /* postscript level to use: affects only compression */
@@ -92,7 +87,7 @@ struct info_s {
   double blacklevel; /* 0 to 1: black/white cutoff in input file */
   int invert;        /* invert bitmap? */
   int opaque;        /* paint white shapes opaquely? */
-  int group;         /* group paths together? */
+  int grouping;      /* 0=flat; 1=connected components; 2=hierarchical */
   int progress;      /* should we display a progress bar? */
   progress_bar_t *progress_bar;  /* which progress bar to use */
 };
@@ -107,7 +102,7 @@ struct imginfo_s {
   double width;        /* desired width of image (in pt or pixels) */
   double height;       /* desired height of image (in pt or pixels) */
   double lmar, rmar, tmar, bmar;   /* requested margins (in pt) */
-  rect_t trans;        /* specify relative position of a tilted rectangle */
+  trans_t trans;        /* specify relative position of a tilted rectangle */
 };
 typedef struct imginfo_s imginfo_t;
 
